﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Priprema
{
    public class PersonManager
    {
        private IRepo repo;
        private IDictionary<string, Person> persons;
        public event OnErrorDelegate OnError;
        public event OnLoadDelegate OnLoad;
        public PersonManager()
        {
            repo = RepoFactory.GetRepo;
               
        }

        public IDictionary<string, Person> Persons {
            get {
                if (persons == null)
                {
                    Load();
                }
                return new Dictionary<string, Person>(persons);
            }
        }

        private void Load()
        {
            try
            {
                IList<Person> list = repo.Load();
                Process(list);
            }
            catch (Exception ex) {
                OnError?.Invoke(this, new OnErrorArgs { Exception = ex });
            } 
        }

        private void Process(IList<Person> list)
        {
            persons = new Dictionary<string, Person>();
            
            foreach (Person person in list) {
                try
                {
                    IList<string> missing = new List<string>();
                    if (person.HasValidOib()) {
                        CheckMissing(person, missing);
                        persons.Add(person.OIB, person);
                        OnLoad?.Invoke(this, new OnLoadArgs { Person = person, Missing = missing });
                    }
                    else
                    {
                        throw new OibException();
                    }
                }
                catch (OibException ex) {
                    OnError?.Invoke(this, new OnErrorArgs { Exception=ex });
                }
            }
        }

        private void CheckMissing(Person person, IList<string> missing)
        {
            if (string.IsNullOrEmpty(person.FirstName))
            {
                missing.Add("FirstName");
            }
            if (string.IsNullOrEmpty(person.LastName))
            {
                missing.Add("LastName");
            }
            if (string.IsNullOrEmpty(person.Email))
            {
                missing.Add("Email");
            }
            if (string.IsNullOrEmpty(person.Phone))
            {
                missing.Add("Phone");
            }
        }
    }
}
